import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import os
import joblib

from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split
from sklearn.metrics import classification_report, confusion_matrix


df = pd.read_csv("../Dataset/Linhac24-25_Sportlogiq.csv", encoding="utf-8")


df['is_goal'] = (df['eventname'] == 'goal').astype(int)

#选择特征
features = [
    'xg_allattempts', 'compiledgametime', 'period', 'scoredifferential',
    'teamskatersonicecount', 'opposingteamskatersonicecount',
    'xadjcoord', 'yadjcoord',
    'teaminpossession', 'currentpossession', 'manpowersituation',
    'ishomegame', 'playerprimaryposition', 'type', 'outcome'
]

df = df[features + ['is_goal']].copy()


df.fillna({'xg_allattempts': 0, 'compiledgametime': 0, 'scoredifferential': 0,
           'teamskatersonicecount': 5, 'opposingteamskatersonicecount': 5,
           'xadjcoord': 0, 'yadjcoord': 0}, inplace=True)


categorical_cols = ['teaminpossession', 'currentpossession', 'manpowersituation',
                    'ishomegame', 'playerprimaryposition', 'type', 'outcome']
df = pd.get_dummies(df, columns=categorical_cols, drop_first=True)

#拆分数据集
X = df.drop(columns=['is_goal'])
y = df['is_goal']

X_train, X_test, y_train, y_test = train_test_split(
    X, y, test_size=0.2, stratify=y, random_state=42
)

#模型路径
model_dir = os.path.join(os.path.dirname(__file__), "../Model")
os.makedirs(model_dir, exist_ok=True)
model_path = os.path.join(model_dir, "RF1.pkl")


if os.path.exists(model_path):
    print("模型已存在，正在加载模型...")
    rf = joblib.load(model_path)
else:
    print("正在训练模型...")
    rf = RandomForestClassifier(n_estimators=100, random_state=42)
    rf.fit(X_train, y_train)
    joblib.dump(rf, model_path)
    print(f"模型已保存到: {model_path}")

#模型评估
y_pred = rf.predict(X_test)

print("\n=== Classification Report ===")
print(classification_report(y_test, y_pred))

print("=== Confusion Matrix ===")
print(confusion_matrix(y_test, y_pred))

importances = pd.Series(rf.feature_importances_, index=X.columns)
top_importances = importances.sort_values(ascending=False).head(20)

plt.figure(figsize=(10, 6))
top_importances.plot(kind='barh')
plt.gca().invert_yaxis()
plt.title("Top 20 Important Features for Predicting Goals")
plt.xlabel("Feature Importance")
plt.tight_layout()

plot_path = os.path.join("../Graph", "RF1.jpg")
plt.savefig(plot_path, dpi=300, bbox_inches='tight')
print(f"图表已保存到: {plot_path}")

plt.show()