import pandas as pd
import joblib
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.model_selection import train_test_split
from sklearn.ensemble import RandomForestClassifier
from sklearn.metrics import classification_report, confusion_matrix
from sklearn.preprocessing import LabelEncoder


df = pd.read_csv("../Dataset/Linhac24-25_Sportlogiq.csv", encoding="utf-8")

df['is_goal'] = (df['eventname'] == 'goal').astype(int)

drop_cols = ['gameid', 'playerid', 'eventname', 'compiledgametime']
df.drop(columns=drop_cols, inplace=True, errors='ignore')

categorical_cols = df.select_dtypes(include='object').columns
for col in categorical_cols:
    df[col] = LabelEncoder().fit_transform(df[col].astype(str))

X = df.drop(columns=['is_goal'])
y = df['is_goal']

X_train, X_test, y_train, y_test = train_test_split(
    X, y, test_size=0.2, stratify=y, random_state=42
)

model = RandomForestClassifier(
    n_estimators=100,
    random_state=42,
    class_weight='balanced',
    n_jobs=-1
)
model.fit(X_train, y_train)

model = joblib.load('../Model/RF2.pkl')
y_pred = model.predict(X_test)

print("=== Classification Report ===")
print(classification_report(y_test, y_pred, digits=4))

print("\n=== Confusion Matrix ===")
print(confusion_matrix(y_test, y_pred))

# 特征重要性
importances = pd.Series(model.feature_importances_, index=X.columns)
print("\n=== Top 10 Important Features ===")
print(importances.sort_values(ascending=False).head(10))


goal_data = df[df['is_goal'] == 1]

#KDE 热力图
plt.figure(figsize=(10, 6))
sns.kdeplot(
    x=goal_data['xadjcoord'],
    y=goal_data['yadjcoord'],
    cmap="Reds",
    fill=True,
    thresh=0.05,
    bw_adjust=1
)
plt.axvline(x=89, color='blue', linestyle='--', label='Goal Line')
plt.xlim(-100, 100)
plt.ylim(-50, 50)
plt.title('Goal Density Heatmap (KDE)')
plt.xlabel('X Coordinate')
plt.ylabel('Y Coordinate')
plt.grid(True)
plt.legend()

plt.tight_layout()
plt.savefig("../Graph/goal_heatmap_KDE.png", dpi=300, bbox_inches='tight')
print("KDE 热力图已保存为: ../Model/goal_heatmap_KDE.png")

plt.show()

# Hexbin 热力图
plt.figure(figsize=(10, 6))
plt.hexbin(
    goal_data['xadjcoord'],
    goal_data['yadjcoord'],
    gridsize=30,
    cmap='Reds',
    mincnt=1
)
plt.colorbar(label='Goal Count')
plt.axvline(x=89, color='blue', linestyle='--', label='Goal Line')
plt.xlim(-100, 100)
plt.ylim(-50, 50)
plt.title('Goal Density Heatmap (Hexbin)')
plt.xlabel('X Coordinate')
plt.ylabel('Y Coordinate')
plt.grid(True)
plt.legend()

plt.tight_layout()
plt.savefig("../Graph/goal_heatmap_Hexbin.png", dpi=300, bbox_inches='tight')
print("Hexbin 热力图已保存为: ../Model/goal_heatmap_Hexbin.png")

plt.show()
