import pandas as pd
import numpy as np
import xgboost as xgb
import matplotlib.pyplot as plt
import os
import joblib
from sklearn.model_selection import train_test_split
from sklearn.metrics import classification_report, confusion_matrix

plt.rcParams['font.family'] = 'Microsoft YaHei'
plt.rcParams['axes.unicode_minus'] = False

df = pd.read_csv("../Dataset/Linhac24-25_Sportlogiq.csv")

df["is_goal"] = (df["eventname"] == "goal").astype(int)

feature_cols = [
    "xadjcoord", "yadjcoord", "compiledgametime",
    "scoredifferential", "teamskatersonicecount", "opposingteamskatersonicecount",
    "ishomegame", "teaminpossession", "currentpossession", "xg_allattempts",
    "type", "manpowersituation", "playerprimaryposition", "outcome"
]

df["xg_allattempts"] = df["xg_allattempts"].fillna(0)
df["compiledgametime"] = df["compiledgametime"].fillna(0)

# One-hot 编码
df_encoded = pd.get_dummies(df[feature_cols], drop_first=True)

#特征
X = df_encoded
y = df["is_goal"]

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

model_dir = os.path.join(os.path.dirname(__file__), "../Model")
os.makedirs(model_dir, exist_ok=True)
model_path = os.path.join(model_dir, "XGB.pkl")

if os.path.exists(model_path):
    print("已检测到已有模型，正在加载中...")
    model = joblib.load(model_path)
else:
    print("正在训练 XGBoost 模型...")
    model = xgb.XGBClassifier(eval_metric='logloss', use_label_encoder=False)
    model.fit(X_train, y_train)
    joblib.dump(model, model_path)
    print(f"模型已保存到: {model_path}")

y_pred = model.predict(X_test)
print("\n=== 分类报告 ===")
print(classification_report(y_test, y_pred))
print("=== 混淆矩阵 ===")
print(confusion_matrix(y_test, y_pred))


plt.figure(figsize=(12, 8))
xgb.plot_importance(model, height=0.6, max_num_features=20, importance_type='gain')
plt.title("XGBoost Feature Importance (by Gain)")  # 英文标题
plt.tight_layout()


importance_plot_path = os.path.join("../Graph", "xgb_feature_importance.png")
plt.savefig(importance_plot_path, dpi=300, bbox_inches='tight')
print(f"📊 Feature importance plot saved to: {importance_plot_path}")

plt.show()
