import pandas as pd
import numpy as np
import seaborn as sns
import xgboost as xgb
import matplotlib.pyplot as plt
from sklearn.model_selection import train_test_split
from sklearn.metrics import classification_report, confusion_matrix

plt.rcParams['font.family'] = 'Microsoft YaHei'
plt.rcParams['axes.unicode_minus'] = False

df = pd.read_csv("../Dataset/Linhac24-25_Sportlogiq.csv")

df["is_goal"] = (df["eventname"] == "goal").astype(int)


feature_cols = [
    "xadjcoord", "yadjcoord", "compiledgametime",
    "scoredifferential", "teamskatersonicecount", "opposingteamskatersonicecount",
    "ishomegame", "teaminpossession", "currentpossession", "xg_allattempts",
    "type", "manpowersituation", "playerprimaryposition", "outcome"
]

df["xg_allattempts"] = df["xg_allattempts"].fillna(0)
df["compiledgametime"] = df["compiledgametime"].fillna(0)

df_encoded = pd.get_dummies(df[feature_cols], drop_first=True)

X = df_encoded
y = df["is_goal"]

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=42)

model = xgb.XGBClassifier(eval_metric='logloss')
model.fit(X_train, y_train)

#model.save_model("../Model/XGB2.pkl")

loaded_model = xgb.XGBClassifier()
loaded_model.load_model("../Model/XGB2.pkl")

y_pred = loaded_model.predict(X_test)
print(classification_report(y_test, y_pred))
print("混淆矩阵：")
print(confusion_matrix(y_test, y_pred))


y_prob = loaded_model.predict_proba(X_test)[:, 1]

#组合 df_test 用于分析
df_test = X_test.copy()
df_test['is_goal'] = y_test
df_test['goal_prob'] = y_prob
df_test['xadjcoord'] = df.loc[df_test.index, 'xadjcoord']
df_test['yadjcoord'] = df.loc[df_test.index, 'yadjcoord']
df_test['manpowersituation'] = df.loc[df_test.index, 'manpowersituation']
df_test['scoredifferential'] = df.loc[df_test.index, 'scoredifferential']

#进球位置与进球概率关系图
plt.figure(figsize=(10, 6))
plt.scatter(df_test['xadjcoord'], df_test['yadjcoord'], c=df_test['goal_prob'], cmap='coolwarm', alpha=0.5)
plt.colorbar(label="Goal Probability")
plt.title("Relationship Between Goal Location and Goal Probability")
plt.xlabel("xadjcoord")
plt.ylabel("yadjcoord")
plt.axhline(0, color='gray', linestyle='--')
plt.axvline(0, color='gray', linestyle='--')
plt.tight_layout()
plt.savefig("../Graph/goal_position_probability.png", dpi=300, bbox_inches='tight')
plt.show()



#局势 vs 进球概率
plt.figure(figsize=(12, 6))
sns.boxplot(x=df_test['manpowersituation'], y=df_test['goal_prob'])
plt.title("Goal Probability Across Different Manpower Situations")
plt.xlabel("Manpower Situation")
plt.ylabel("Goal Probability")
plt.xticks(rotation=45)
plt.tight_layout()
plt.savefig("../Graph/goal_prob_by_manpowersituation.png", dpi=300, bbox_inches='tight')
plt.show()


# 比分差 vs 进球概率
plt.figure(figsize=(10, 6))
sns.lineplot(x=df_test['scoredifferential'], y=df_test['goal_prob'], marker='o')
plt.title("Goal Probability vs Score Differential")
plt.xlabel("Score Differential (Team - Opponent)")
plt.ylabel("Goal Probability")
plt.tight_layout()
plt.savefig("../Graph/goal_prob_by_scorediff.png", dpi=300, bbox_inches='tight')
plt.show()


